<?php
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Slim\Factory\AppFactory;
use Slim\Views\Twig;
use Slim\Views\TwigMiddleware;
use DI\Container;
use Slim\Routing\RouteContext;
use Slim\Routing\RouteCollectorProxy;
use Psr\Http\Server\RequestHandlerInterface;
use Psr\Http\Message\UploadedFileInterface;
use App\Middleware\LoginAuthMiddleware;
use Slim\Middleware\MethodOverrideMiddleware;

require __DIR__ . '/vendor/autoload.php';
/* require_once __DIR__ . '/config/config.php'; */

try {
    require '/persist/etc/casper/config.php';
} catch (Error $ex) {
    require_once __DIR__ . '/config/config.php';
}


/**
 * Instantiate App
 *
 * In order for the factory to work you need to ensure you have installed
 * a supported PSR-7 implementation of your choice e.g.: Slim PSR-7 and a supported
 * ServerRequest creator (included with Slim PSR-7)
 */

$container = new Container();
AppFactory::setContainer($container);

$app = AppFactory::create();

$container->set('settings', ['ldap' => $config['ldap'],
			     'media' => $config['media'],
			     'rest' => $config['rest'],
			     'defaults' => $config['defaults'],
			     'call_status' => $config['call_status']
]);

/**
 * The routing middleware should be added earlier than the ErrorMiddleware
 * Otherwise exceptions thrown from it will not be handled by the middleware
 */
// Parse json, form data and xml
$app->add(LoginauthMiddleware::class);
$app->addBodyParsingMiddleware();
$app->addRoutingMiddleware();
$app->add(new MethodOverrideMiddleware);
$twig = Twig::create('./public', ['cache' => false]);
$app->add(TwigMiddleware::create($app, $twig));


/* $app->add(\App\Middleware\RenderMiddleware::class); */

/**
 * Add Error Middleware
 *
 * @param bool                  $displayErrorDetails -> Should be set to false in production
 * @param bool                  $logErrors -> Parameter is passed to the default ErrorHandler
 * @param bool                  $logErrorDetails -> Display error details in error log
 * @param LoggerInterface|null  $logger -> Optional PSR-3 Logger  
 *
 * Note: This middleware should be added last. It will not handle any exceptions/errors
 * for middleware added after it.
 */
$errorMiddleware = $app->addErrorMiddleware(true, true, true);

$app->get('/ui',                'App\Modules\Main\Main:main');
$app->get('/ui/dict',            'App\Modules\User\User:departs');
$app->get('/ui/dict/{name}',     'App\Modules\User\User:depart');
$app->get('/ui/search',      'App\Modules\User\User:search');

$app->group('/ui/user', function (RouteCollectorProxy $group){
    $group->get('/{name}',     'App\Modules\User\User:get_user');
    $group->get('/delete/{name}',  'App\Modules\User\User:del_user');
    $group->post('/{name}',    'App\Modules\User\User:edit_user'); 
});

$app->group('/ui/depart', function (RouteCollectorProxy $group){
    $group->post('/new',  'App\Modules\User\User:create_depart');
    $group->post('/modify',  'App\Modules\User\User:modify_depart');
    $group->post('/update/{name}',  'App\Modules\User\User:update_depart');
    $group->get('/delete/{name}',  'App\Modules\User\User:del_depart');
});

$app->group('/ui/group', function (RouteCollectorProxy $group){
    $group->get('',           'App\Modules\User\Group:get');
    $group->get('/{name}',    'App\Modules\User\Group:group');
    $group->post('/new',      'App\Modules\User\Group:new');
    $group->post('/modify',   'App\Modules\User\Group:list_modify');
    $group->post('/{name}',   'App\Modules\User\Group:add_to_list');
    $group->get('/delete/{name}',   'App\Modules\User\Group:del_list');
});

$app->group('/ui/export', function (RouteCollectorProxy $group){
    $group->get('',          'App\Modules\User\User:export_load_file');
    $group->post('',         'App\Modules\User\User:export_parse_file');
});

$app->group('/ui/session', function (RouteCollectorProxy $group){
    $group->get('',         'App\Modules\Session\Session:get');
    $group->post('',        'App\Modules\Session\Session:new');
    $group->get('/test_call',  'App\Modules\Session\Session:test_call');
    $group->get('/{uuid}',  'App\Modules\Session\Session:status');
    $group->post('/{uuid}',  'App\Modules\Session\Session:start');
    $group->get('/{uuid}/config',  'App\Modules\Session\Session:config');
    $group->post('/{uuid}/config',  'App\Modules\Session\Session:update_config');
    $group->get('/{uuid}/dict',  'App\Modules\Session\Session:dict');
    $group->get('/{uuid}/dict/{depart}',  'App\Modules\Session\Session:depart');
    $group->get('/{uuid}/search',  'App\Modules\Session\Session:search');
    $group->get('/{uuid}/group',  'App\Modules\Session\Session:groups');
    $group->get('/{uuid}/group/{group}',  'App\Modules\Session\Session:group');
    $group->get('/{uuid}/report',  'App\Modules\Archive\Report:report');
    $group->post('/{uuid}/subscriber',  'App\Modules\Session\Session:subscriber');
    $group->post('/{uuid}/depart',  'App\Modules\Session\Session:add_depart');
    $group->post('/{uuid}/group',  'App\Modules\Session\Session:add_group');
    $group->get('/{uuid}/media',  'App\Modules\Session\Session:media');
});

$app->group('/ui/config', function (RouteCollectorProxy $group){
    $group->get('',              'App\Modules\Config\Config:get');
    $group->post('/call_status', 'App\Modules\Config\Config:call_status');
    $group->post('',             'App\Modules\Config\Config:save');
});

$app->group('/ui/media', function (RouteCollectorProxy $group){
    $group->get('',           'App\Modules\Media\Media:get');
    $group->post('',           'App\Modules\Media\Media:upload');
    $group->get('/delete/{name}', 'App\Modules\Media\Media:del');
    $group->post('/record',   'App\Modules\Media\Media:record');
    $group->get('/express', 'App\Modules\Media\Media:express');
});

$app->group('/ui/phrase', function (RouteCollectorProxy $group){
    $group->get('',           'App\Modules\Phrase\Phrase:get_phrase');
    $group->post('',           'App\Modules\Phrase\Phrase:edit_phrase');
});

$app->group('/ui/archive', function (RouteCollectorProxy $group){
    $group->get('',          'App\Modules\Archive\Report:archive_list');
    $group->get('/{name}',          'App\Modules\Archive\Report:archive_show');
    $group->delete('/{name}',          'App\Modules\Archive\Report:delete_archive');
    $group->post('/{uuid}',   'App\Modules\Archive\Report:archive');
});

$app->get('/ui/update-date',     'App\Modules\Main\Main:update_date');
$app->get('/ui/state',            'App\Modules\Main\Main:current_state');
$app->get('/ui/report/{name}',    'App\Modules\Archive\Report:report');
$app->post('/ui/system/{cmd}',    'App\Modules\Main\Main:system');


$app->group('/ui/operator', function (RouteCollectorProxy $group){
    $group->get('',          'App\Modules\User\Operator:list');
    $group->get('/{name}', 'App\Modules\User\Operator:edit');
    $group->delete('/{name}', 'App\Modules\User\Operator:delete') ;
    $group->post('/{name}',          'App\Modules\User\Operator:create');
    /* $group->get('/delete/{name}', 'App\Modules\User\Operator:delete'); */
    
});


$app->get('/ui/login', 'App\Modules\Login\Login:enter_page');
$app->post('/ui/login', 'App\Modules\Login\Login:login');
$app->get('/ui/logout', 'App\Modules\Login\Login:logout');


$app->post('/ui/select', 'App\Modules\Action\Action:select');

$app->get('/ui/update', 'App\Modules\Main\Main:update');
$app->post('/ui/update', 'App\Modules\Main\Main:load_image');
$app->post('/ui/update/image', 'App\Modules\Main\Main:update_image');

$app->get('/ui/update/restart', function (Request $request, Response $response, $args) {
    $params = $request->getParsedBody();
    $resp = shell_exec("sudo /var/www/cgi/update_restart 2>&1 ");

    return $response->withStatus(302)
		    ->withHeader('Location', "/ui");
});

$app->post('/uids/export', function (Request $request, Response $response, $args) {
    $view = Twig::fromRequest($request);
    $directory = "/tmp";
    $uploadedFiles = $request->getUploadedFiles();

    // handle single input with single file upload
    $uploadedFile = $uploadedFiles['phones'];
    if ($uploadedFile->getError() === UPLOAD_ERR_OK) {
	$filename = moveUploadedFile($directory, $uploadedFile);
	$response->getBody()->write('Uploaded: ' . $filename . '<br/>');
    }

    $db = new SQLite3("/tmp/".$filename);
    $q = 'SELECT u.id, g.group_name, p.poss_name, u.first_name, u.second_name, u.family_name, u.numbers, clist.gr
FROM phonedir as dir
LEFT JOIN (select u1.id, u1.first_name, u1.second_name, u1.family_name, group_concat(ph.number, \';\') numbers
from users as u1
left join phones as ph on ph.user_id_id = u1.id
group by u1.id) as u 
on u.id = dir.user_id_id
left join (select c.record_id_id id, group_concat(list.list_name, \';\') gr
from customlist as c
left join list on list_id_id = list.id group by record_id_id) as clist on clist.id = dir.user_id_id
left join possitions as p on poss_id_id = p.id
left join "group" as g on group_id_id = g.id;';

    $result = $db->query($q);

    $count = 0;
    $phonedir = [];
    $custom_list = [];
    while ($row = $result->fetchArray()) {
	$count = $count + 1;
	$groups = explode(';', $row['gr']);
	$phonedir[$row['group_name']][] =
	    array("name" => $row['first_name'],
		  "family" => $row['family_name'],
		  "second" => $row['second_name'],
		  "poss" => $row['poss_name'],
		  "numbers" => explode(';', $row['numbers']),
		  "lists" => $groups);
	foreach ($groups as $l){
	    $custom_list[$l][] = array("name" => $row['first_name'],
				       "family" => $row['family_name'],
				       "second" => $row['second_name']);
	}
    }
    $db->close();
    /* echo var_dump($phonedir); */
    return $view->render($response, 'export.html',
			 [ 'phonedir' => $phonedir,
			   'custom_list' => $custom_list,
			   'filename' => $filename
    ]);
});


function fs_link($addr) {
    return '/' . $addr;
}

/* function status() {
 *     $url = "http://127.0.0.1/api/v1/status";
 *     $ch = curl_init();
 *     curl_setopt($ch, CURLOPT_URL, $url);
 *     curl_setopt($ch, CURLOPT_PORT, 9000);
 *     curl_setopt($ch, CURLOPT_HEADER, 0);
 *     curl_setopt($ch, CURLOPT_NOBODY, 0);
 *     curl_setopt($ch, CURLOPT_RETURNTRANSFER, true); // Return, do not echo result 
 *     curl_setopt($ch, CURLOPT_HTTPHEADER, array("Accept: application/json"));
 *     $status = curl_exec($ch);
 *     $info = curl_getinfo($ch);
 *     curl_close($ch);
 *     return json_decode($status);
 * 
 * } */

function renderTemplate($request, $response, $template, $data) {
    $view = Twig::fromRequest($request);
    date_default_timezone_set("Europe/Minsk");
    $data['current_date'] = date("d-m-Y H:i:s");
    return $view->render($response, $template, $data);
}

// Run app
$app->run();

